function metis_vignetting, data, header, cal_pack, error = error, quality_matrix = quality_matrix, history = history

	journal, 'Vignetting correction:'

	if header.filter.contains('VL', /fold) then vig_info = cal_pack.vl_channel.vignetting
	if header.filter.contains('UV', /fold) then vig_info = cal_pack.uv_channel.vignetting

	vig_image = readfits(cal_pack.path + vig_info.file_name, /silent)
	vig_error = readfits(cal_pack.path + vig_info.error, /silent)
	vig_image = median(vig_image, 5)
	vig_file = vig_info.file_name

	dx = 0.
	dy = 0.

	if header.filter.contains('UV', /fold) then begin
		vl_boresight = cal_pack.vl_channel.boresight[0]
		uv_boresight = cal_pack.uv_channel.boresight[0]

		x1 = uv_boresight.borpix1.value
		x2 = vl_boresight.borpix1.value
		y1 = uv_boresight.borpix2.value
		y2 = vl_boresight.borpix2.value

		plate_scale = vl_boresight.plate_scale.value
		dx = (x1 - x2)/plate_scale
		dy = (y1 - y2)/plate_scale

		vig_image = rotate(vig_image, 1)
		vig_image = shift(vig_image, dx, dy)
		vig_image[2047 - abs(dx) : 2047, *] = 0.
		vig_image[*, 0 : abs(dy) - 1] = 0.
		vig_image = rotate(vig_image, 3)
		vig_image = reverse(vig_image, 1)

		; WARN - error for the shift of the uv vignetting function must be included

		vig_error = rotate(vig_error, 1)
		vig_error = shift(vig_error, dx, dy)
		vig_error[2047 - abs(dx) : 2047, *] = 0.
		vig_error[*, 0 : abs(dy) - 1] = 0.
		vig_error = rotate(vig_error, 3)
		vig_error = reverse(vig_error, 1)

		journal, '  UV shift applied = [' + string(dx, format = '(F0.1)') + ', ' + string(dy, format = '(F0.1)') + '] pixel'
	endif

	; WARN - error for binning must be checked

	nbin = header.nbin
	vig_image = rebin(vig_image, header.naxis1, header.naxis2)
	vig_error = sqrt(rebin(vig_error^2, header.naxis1, header.naxis2) * nbin)/nbin

	vig_image = (vig_image > 0.) < 1.
	s = where(vig_image eq 0. or vig_image eq 1., count)
	if count gt 0 then vig_image[s] = 1.
	data = data/vig_image
	
	if count gt 0 then data[s] = 0.

	if not isa(error) then error = 0.
	error += (vig_error/vig_image)^2

	if isa(quality_matrix) then $
		if count gt 0 then quality_matrix[s] = 0
	
	if ~ isa(history) then history = !null
	history = [history, 'Vignetting correction: ', '  ' + vig_file + ' shifted by [' + string(dx, format = '(F0.1)') + ', ' + string(dy, format = '(F0.1)') + '] pixel']

	journal, '  vign. function file = ' + vig_file

	return, data
end
